import socket
import sys
from xml.sax import saxutils
from xml.sax import make_parser
from xml.sax.handler import feature_namespaces
import toolbox
import hashlib
import time

class Toodledo:
    "Class to interface with toodledo.com"
    HOST = 'www.toodledo.com'
    PORT = 80
    hash = None
    userID = None
    password = None
    sock = None
    timestamp = 0

    def __init__(self, username, password):
        #logging.debug("Initializing the toodledo object")
        self.userID = username
        self.password = password
        self.projlist = None
        self.conlist = None
        self.timestamp = 0

        url = "http://www.toodledo.com/api.php?method=getToken;userid="+username
        errorResponse = '<token>None</token>'
        self.loginResponse = toolbox.loadXML(url, errorResponse)
        token = self.loginResponse.firstChild.data
        if token != "None":
           mp = hashlib.md5(password)
           #logging.debug('password is '+password +', token is '+token+", hash(password) is "+mp.hexdigest())
           m = hashlib.md5(mp.hexdigest()+token+username)
           self.hash = m.hexdigest()
           self.timestamp = time.time()
        else:
           self.hash = "None"

        #logging.debug( self.hash )

    def init(self, username, password):
        #logging.debug("Initializing Toodledo module")
        self.userID = username
        self.password = password
        url = "http://www.toodledo.com/api.php?method=getToken;userid="+username
        errorResponse = '<token>None</token>'
        self.loginResponse = toolbox.loadXML(url, errorResponse)
        token = self.loginResponse.firstChild.data
        #logging.debug( token )
        if token != "None":
           mp = hashlib.md5(password)
           #logging.debug( 'password is '+password +', token is '+token+", hash(password) is "+mp.hexdigest())
           m = hashlib.md5(mp.hexdigest()+token+username)
           self.hash = m.hexdigest()
           self.timestamp = time.time()
        else:
           self.hash = "None"

        #logging.debug( self.hash )

    def getHash(self):
        return self.hash
        
    def getProjects(self, possibleID=None, possibleHash=None):
        tokenAge = time.time() - self.timestamp
        if (self.hash == "None" or tokenAge > 3600):
           print("Reinitializing the Toodledo object")
           self.init(self.userID, self.password) 
        specHash = self.hash
        if possibleHash != None: specHash = possibleHash
        specID = self.userID
        if possibleID != None: specID = possibleID
        
        url = 'http://www.toodledo.com/api.php?method=getFolders;key='+specHash
        projResponse = toolbox.loadXML(url)
        projstr = ""
        self.fidlist = []
        for proj in projResponse.getElementsByTagName("folder"):
            fid = proj.attributes["id"].value
            self.fidlist.append(fid)
            projstr += proj.firstChild.data
            projstr += "|"
        self.projlist = projstr.split('|')
        print self.fidlist
        
        return self.projlist
        
    def getContexts(self, possibleID=None, possibleHash=None):
        tokenAge = time.time() - self.timestamp
        if (self.hash == "None" or tokenAge > 3600):
           print("Reinitializing the Toodledo object")
           self.init(self.userID, self.password) 
        specHash = self.hash
        if possibleHash != None: specHash = possibleHash
        specID = self.userID
        if possibleID != None: specID = possibleID
        
        url = 'http://www.toodledo.com/api.php?method=getContexts;key='+specHash
        projResponse = toolbox.loadXML(url)
        projstr = ""
        self.cidlist = []
        for proj in projResponse.getElementsByTagName("context"):
            fid = proj.attributes["id"].value
            self.cidlist.append(fid)
            projstr += proj.firstChild.data
            projstr += "|"
        self.conlist = projstr.split('|')
        
        return self.conlist
        
    def saveTask(self, taskDesc, projName, priority, duedate, conName):
        #logging.debug("Saving to Toodledo")
        specHash = self.hash
        specID = self.userID

        index=0
        cindex=0
        #logging.debug("Getting the project list")
        if (self.projlist == None): self.getProjects()
        for proj in self.projlist:
           if proj == projName:
              break
           index += 1
        #logging.debug("Getting the context list")
        if (self.conlist == None): self.getContexts()
        for con in self.conlist:
           if con == conName:
              break
           cindex += 1

        taskDesc = taskDesc.replace(" ","%20")
        url = 'http://www.toodledo.com/api.php?method=addTask;key='+specHash+';title='+taskDesc+';priority='+priority+';folder='+self.fidlist[index]+';context='+self.cidlist[cindex]
        #logging.debug("URL is "+url)
        if duedate != "":
           url += ";duedate="+duedate

        projResponse = toolbox.loadXML(url)
