from xml.dom import minidom

def openAnything(source, alternative=None):            
    """URI, filename, or string --> stream

    This function lets you define parsers that take any input source
    (URL, pathname to local or network file, or actual data as a string)
    and deal with it in a uniform manner.  Returned object is guaranteed
    to have all the basic stdio read methods (read, readline, readlines).
    Just .close() the object when you're done with it.
    
    Examples:
    >>> from xml.dom import minidom
    >>> sock = openAnything("http://localhost/kant.xml")
    >>> doc = minidom.parse(sock)
    >>> sock.close()
    >>> sock = openAnything("c:\\inetpub\\wwwroot\\kant.xml")
    >>> doc = minidom.parse(sock)
    >>> sock.close()
    >>> sock = openAnything("<ref id='conjunction'><text>and</text><text>or</text></ref>")
    >>> doc = minidom.parse(sock)
    >>> sock.close()
    """
    if hasattr(source, "read"):
        return source

    if source == '-':
        import sys
        return sys.stdin

    # try to open with urllib (if source is http, ftp, or file URL)
    import urllib                         
    try:                                  
        return urllib.urlopen(source)     
    except (IOError, OSError):            
        ##pass                             
        print "ERROR with URL ("+source+")!\n"
    
    # try to open with native open function (if source is pathname)
    try:                                  
        return open(source)               
    except (IOError, OSError):            
        ##pass                              
        print "ERROR with file!\n"
    
    # treat source as string
    import StringIO 
    if alternative == None:  
        print 'LAST RESORT.  String is "'+source+'"\n'                    
        return StringIO.StringIO(str(source)) 
    else:
        print 'LAST RESORT.  String is "'+alternative+'"\n'  
        return StringIO.StringIO(str(alternative)) 
    
def loadXML(source, alternative=None):
    """load XML input source, return parsed XML document

    - a URL of a remote XML file ("http://diveintopython.org/kant.xml")
    - a filename of a local XML file ("~/diveintopython/common/py/kant.xml")
    - standard input ("-")
    - the actual XML document, as a string
    """
    sock = openAnything(source, alternative)
    try:
        xmldoc = minidom.parse(sock).documentElement
    except (IOError, OSError):
        print "ERROR with data"
        sock.close()
        sock = openAnything('<response method="getProjects"><project projName="ERROR!"/></response>');
        xmldoc = minidom.parse(sock).documentElement
    sock.close()
    return xmldoc
